<?php

declare(strict_types=1);

namespace Drupal\a12s_theme_builder\Block;

use Drupal\block\BlockInterface;
use Drupal\Core\Form\SubformStateInterface;

/**
 * Provides options to configure the display of the language switcher block.
 */
class LanguageSwitcher extends SettingsAlterBase {

  /**
   * {@inheritdoc}
   */
  public function defaultValues(): array {
    return [
      'display' => 'list',
      'use_langcode' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function applies(BlockInterface $entity): bool {
    return str_starts_with($entity->getPluginId(), 'language_block:');
  }

  /**
   * {@inheritdoc}
   */
  public function configurationKey(): string {
    return 'language_switcher';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array &$form, SubformStateInterface $formState, array $settings = []): void {
    $this->mergeWithDefault($settings);
    $form['display'] = [
      '#type' => 'select',
      '#title' => $this->t('Display style'),
      '#options' => [
        'dropdown' => $this->t('Dropdown'),
        'list' => $this->t('Bullet list'),
      ],
      '#default_value' => $settings['display'],
    ];

    $form['use_langcode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use the language code instead of the name'),
      '#default_value' => $settings['use_langcode'],
    ];
  }

  /**
   * Preprocess a "language switcher" block.
   *
   * @see a12s_theme_builder_preprocess_block()
   */
  public function preprocessBlock(array &$variables, array $settings): void {
    $this->mergeWithDefault($settings);

    // @todo how to know if we deal with "content" or "interface" language?
    $currentLanguage = \Drupal::languageManager()->getCurrentLanguage('language_content');
    $activeLanguageId = $currentLanguage->getId();
    // @todo move this to $variables['content']?
    $variables['use_dropdown'] = $settings['display'] === 'dropdown';

    if ($variables['use_dropdown']) {
      $variables['dropdown_title'] = $currentLanguage->getName();

      if ($settings['use_langcode']) {
        $variables['dropdown_title'] = $activeLanguageId;
      }
    }

    // @todo provide some new theme suggestions in the "#theme" key, like
    //   "links__language_block__dropdown" and create the related templates.
    if (!empty($variables['content']['#links'])) {
      foreach ($variables['content']['#links'] as $key => &$link) {
        // Remove the active language from the list when using a dropdown display.
        if ($variables['use_dropdown'] && $key === $activeLanguageId) {
          unset($variables['content']['#links'][$key]);
          continue;
        }

        // Use language ID as title instead of names.
        if ($settings['use_langcode']) {
          $link['title'] = $key;
        }
      }
    }
  }

}
